# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..types.attribute_expanded_response import AttributeExpandedResponse
from ..types.create_attribute_request_entities import CreateAttributeRequestEntities
from ..types.get_attributes_by_entity_type_response import GetAttributesByEntityTypeResponse
from .types.create_attribute_request_attribute_type import CreateAttributeRequestAttributeType
from .types.create_attribute_request_entity_type import CreateAttributeRequestEntityType
from .types.create_attribute_request_unit import CreateAttributeRequestUnit
from .types.delete_attributes_request_entity_type import DeleteAttributesRequestEntityType
from .types.get_attribute_request_entity_type import GetAttributeRequestEntityType
from .types.get_attributes_by_entity_type_request_entity_type import GetAttributesByEntityTypeRequestEntityType
from .types.update_attribute_request_attribute_type import UpdateAttributeRequestAttributeType
from .types.update_attribute_request_entity_type import UpdateAttributeRequestEntityType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAttributesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_attributes_by_entity_type(
        self,
        *,
        entity_type: GetAttributesByEntityTypeRequestEntityType,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetAttributesByEntityTypeResponse]:
        """
        Fetch all attributes in an organization associated with either drivers or assets.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        entity_type : GetAttributesByEntityTypeRequestEntityType
            Denotes the type of entity, driver or asset.

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetAttributesByEntityTypeResponse]
            All attributes in an organization for an entity type
        """
        _response = self._client_wrapper.httpx_client.request(
            "attributes",
            method="GET",
            params={
                "entityType": entity_type,
                "limit": limit,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetAttributesByEntityTypeResponse,
                    parse_obj_as(
                        type_=GetAttributesByEntityTypeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_attribute(
        self,
        *,
        attribute_type: CreateAttributeRequestAttributeType,
        entity_type: CreateAttributeRequestEntityType,
        name: str,
        entities: typing.Optional[typing.Sequence[CreateAttributeRequestEntities]] = OMIT,
        number_values: typing.Optional[typing.Sequence[float]] = OMIT,
        string_values: typing.Optional[typing.Sequence[str]] = OMIT,
        unit: typing.Optional[CreateAttributeRequestUnit] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AttributeExpandedResponse]:
        """
        Creates a new attribute in the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        attribute_type : CreateAttributeRequestAttributeType
            Denotes the data type of the attribute's values. Valid values: `single-select`, `multi-select`, `text`, `freeform-multi-select`, `number`, `date`.

        entity_type : CreateAttributeRequestEntityType
            Denotes the type of entity, driver or asset.

        name : str
            Name

        entities : typing.Optional[typing.Sequence[CreateAttributeRequestEntities]]
            Entities that will be applied to this attribute

        number_values : typing.Optional[typing.Sequence[float]]
            Number values that can be associated with this attribute

        string_values : typing.Optional[typing.Sequence[str]]
            String values that can be associated with this attribute

        unit : typing.Optional[CreateAttributeRequestUnit]
            Unit of the attribute (only for Number attributes).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AttributeExpandedResponse]
            Newly created attribute object with ID.
        """
        _response = self._client_wrapper.httpx_client.request(
            "attributes",
            method="POST",
            json={
                "attributeType": attribute_type,
                "entities": convert_and_respect_annotation_metadata(
                    object_=entities, annotation=typing.Sequence[CreateAttributeRequestEntities], direction="write"
                ),
                "entityType": entity_type,
                "name": name,
                "numberValues": number_values,
                "stringValues": string_values,
                "unit": unit,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AttributeExpandedResponse,
                    parse_obj_as(
                        type_=AttributeExpandedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_attribute(
        self,
        id: str,
        *,
        entity_type: GetAttributeRequestEntityType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AttributeExpandedResponse]:
        """
        Fetch an attribute by id, including all of its applications.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Samsara-provided UUID of the attribute.

        entity_type : GetAttributeRequestEntityType
            Denotes the type of entity, driver or asset.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AttributeExpandedResponse]
            The attribute corresponding to request id.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"attributes/{jsonable_encoder(id)}",
            method="GET",
            params={
                "entityType": entity_type,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AttributeExpandedResponse,
                    parse_obj_as(
                        type_=AttributeExpandedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self,
        id: str,
        *,
        entity_type: DeleteAttributesRequestEntityType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Delete an attribute by id, including all of its applications.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Samsara-provided UUID of the attribute.

        entity_type : DeleteAttributesRequestEntityType
            Denotes the type of entity, driver or asset.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"attributes/{jsonable_encoder(id)}",
            method="DELETE",
            params={
                "entityType": entity_type,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_attribute(
        self,
        id: str,
        *,
        entity_type: UpdateAttributeRequestEntityType,
        attribute_type: typing.Optional[UpdateAttributeRequestAttributeType] = OMIT,
        entities: typing.Optional[typing.Sequence[CreateAttributeRequestEntities]] = OMIT,
        name: typing.Optional[str] = OMIT,
        number_values: typing.Optional[typing.Sequence[float]] = OMIT,
        string_values: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AttributeExpandedResponse]:
        """
        Updates an attribute in the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Samsara-provided UUID of the attribute.

        entity_type : UpdateAttributeRequestEntityType
            Denotes the type of entity, driver or asset.

        attribute_type : typing.Optional[UpdateAttributeRequestAttributeType]
            Denotes the data type of the attribute's values. Valid values: `single-select`, `multi-select`, `text`, `freeform-multi-select`, `number`, `date`.

        entities : typing.Optional[typing.Sequence[CreateAttributeRequestEntities]]
            Entities that will be applied to this attribute

        name : typing.Optional[str]
            Name

        number_values : typing.Optional[typing.Sequence[float]]
            Number values that can be associated with this attribute

        string_values : typing.Optional[typing.Sequence[str]]
            String values that can be associated with this attribute

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AttributeExpandedResponse]
            Newly created attribute object with ID.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"attributes/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "attributeType": attribute_type,
                "entities": convert_and_respect_annotation_metadata(
                    object_=entities, annotation=typing.Sequence[CreateAttributeRequestEntities], direction="write"
                ),
                "entityType": entity_type,
                "name": name,
                "numberValues": number_values,
                "stringValues": string_values,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AttributeExpandedResponse,
                    parse_obj_as(
                        type_=AttributeExpandedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAttributesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_attributes_by_entity_type(
        self,
        *,
        entity_type: GetAttributesByEntityTypeRequestEntityType,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetAttributesByEntityTypeResponse]:
        """
        Fetch all attributes in an organization associated with either drivers or assets.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        entity_type : GetAttributesByEntityTypeRequestEntityType
            Denotes the type of entity, driver or asset.

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetAttributesByEntityTypeResponse]
            All attributes in an organization for an entity type
        """
        _response = await self._client_wrapper.httpx_client.request(
            "attributes",
            method="GET",
            params={
                "entityType": entity_type,
                "limit": limit,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetAttributesByEntityTypeResponse,
                    parse_obj_as(
                        type_=GetAttributesByEntityTypeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_attribute(
        self,
        *,
        attribute_type: CreateAttributeRequestAttributeType,
        entity_type: CreateAttributeRequestEntityType,
        name: str,
        entities: typing.Optional[typing.Sequence[CreateAttributeRequestEntities]] = OMIT,
        number_values: typing.Optional[typing.Sequence[float]] = OMIT,
        string_values: typing.Optional[typing.Sequence[str]] = OMIT,
        unit: typing.Optional[CreateAttributeRequestUnit] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AttributeExpandedResponse]:
        """
        Creates a new attribute in the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        attribute_type : CreateAttributeRequestAttributeType
            Denotes the data type of the attribute's values. Valid values: `single-select`, `multi-select`, `text`, `freeform-multi-select`, `number`, `date`.

        entity_type : CreateAttributeRequestEntityType
            Denotes the type of entity, driver or asset.

        name : str
            Name

        entities : typing.Optional[typing.Sequence[CreateAttributeRequestEntities]]
            Entities that will be applied to this attribute

        number_values : typing.Optional[typing.Sequence[float]]
            Number values that can be associated with this attribute

        string_values : typing.Optional[typing.Sequence[str]]
            String values that can be associated with this attribute

        unit : typing.Optional[CreateAttributeRequestUnit]
            Unit of the attribute (only for Number attributes).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AttributeExpandedResponse]
            Newly created attribute object with ID.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "attributes",
            method="POST",
            json={
                "attributeType": attribute_type,
                "entities": convert_and_respect_annotation_metadata(
                    object_=entities, annotation=typing.Sequence[CreateAttributeRequestEntities], direction="write"
                ),
                "entityType": entity_type,
                "name": name,
                "numberValues": number_values,
                "stringValues": string_values,
                "unit": unit,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AttributeExpandedResponse,
                    parse_obj_as(
                        type_=AttributeExpandedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_attribute(
        self,
        id: str,
        *,
        entity_type: GetAttributeRequestEntityType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AttributeExpandedResponse]:
        """
        Fetch an attribute by id, including all of its applications.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Samsara-provided UUID of the attribute.

        entity_type : GetAttributeRequestEntityType
            Denotes the type of entity, driver or asset.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AttributeExpandedResponse]
            The attribute corresponding to request id.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"attributes/{jsonable_encoder(id)}",
            method="GET",
            params={
                "entityType": entity_type,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AttributeExpandedResponse,
                    parse_obj_as(
                        type_=AttributeExpandedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self,
        id: str,
        *,
        entity_type: DeleteAttributesRequestEntityType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Delete an attribute by id, including all of its applications.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Samsara-provided UUID of the attribute.

        entity_type : DeleteAttributesRequestEntityType
            Denotes the type of entity, driver or asset.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"attributes/{jsonable_encoder(id)}",
            method="DELETE",
            params={
                "entityType": entity_type,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_attribute(
        self,
        id: str,
        *,
        entity_type: UpdateAttributeRequestEntityType,
        attribute_type: typing.Optional[UpdateAttributeRequestAttributeType] = OMIT,
        entities: typing.Optional[typing.Sequence[CreateAttributeRequestEntities]] = OMIT,
        name: typing.Optional[str] = OMIT,
        number_values: typing.Optional[typing.Sequence[float]] = OMIT,
        string_values: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AttributeExpandedResponse]:
        """
        Updates an attribute in the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Samsara-provided UUID of the attribute.

        entity_type : UpdateAttributeRequestEntityType
            Denotes the type of entity, driver or asset.

        attribute_type : typing.Optional[UpdateAttributeRequestAttributeType]
            Denotes the data type of the attribute's values. Valid values: `single-select`, `multi-select`, `text`, `freeform-multi-select`, `number`, `date`.

        entities : typing.Optional[typing.Sequence[CreateAttributeRequestEntities]]
            Entities that will be applied to this attribute

        name : typing.Optional[str]
            Name

        number_values : typing.Optional[typing.Sequence[float]]
            Number values that can be associated with this attribute

        string_values : typing.Optional[typing.Sequence[str]]
            String values that can be associated with this attribute

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AttributeExpandedResponse]
            Newly created attribute object with ID.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"attributes/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "attributeType": attribute_type,
                "entities": convert_and_respect_annotation_metadata(
                    object_=entities, annotation=typing.Sequence[CreateAttributeRequestEntities], direction="write"
                ),
                "entityType": entity_type,
                "name": name,
                "numberValues": number_values,
                "stringValues": string_values,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AttributeExpandedResponse,
                    parse_obj_as(
                        type_=AttributeExpandedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

SafetyEventV2MediaResponseBodyCameraRole = typing.Union[
    typing.Literal[
        "leftMirrorMount",
        "leftSide",
        "rightMirrorMount",
        "rightSide",
        "rearHigh",
        "rearBumper",
        "inCab",
        "front",
        "hopper",
        "other1",
        "other2",
        "other3",
        "other4",
        "leftBev",
        "rightBev",
        "rearBev",
        "frontBev",
        "otherBev",
        "bevNotUsed",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class SafetyScoreConfigurationSettingsObjectResponseBody(UniversalBaseModel):
    """
    The configurable safety infraction weights.
    """

    ai_inattentive_driving_detection_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="aiInattentiveDrivingDetectionWeight"),
        pydantic.Field(
            alias="aiInattentiveDrivingDetectionWeight",
            description="Score weight for AI-detected inattentive driving behavior.",
        ),
    ] = None
    crash_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="crashWeight"),
        pydantic.Field(alias="crashWeight", description="Score weight for crash behavior."),
    ] = None
    defensive_driving_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="defensiveDrivingWeight"),
        pydantic.Field(
            alias="defensiveDrivingWeight",
            description="Score weight for defensive driving behavior. This behavior has a positive impact on the safety score.",
        ),
    ] = None
    did_not_yield_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="didNotYieldWeight"),
        pydantic.Field(alias="didNotYieldWeight", description="Score weight for driver not yielding."),
    ] = None
    drowsy_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="drowsyWeight"),
        pydantic.Field(alias="drowsyWeight", description="Score weight for drowsy behavior."),
    ] = None
    eating_drinking_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="eatingDrinkingWeight"),
        pydantic.Field(alias="eatingDrinkingWeight", description="Score weight for eating/drinking behavior."),
    ] = None
    following_distance_moderate_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="followingDistanceModerateWeight"),
        pydantic.Field(
            alias="followingDistanceModerateWeight",
            description="Score weight for moderate (2-4s) following distance behavior.",
        ),
    ] = None
    following_distance_severe_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="followingDistanceSevereWeight"),
        pydantic.Field(
            alias="followingDistanceSevereWeight",
            description="Score weight for severe (0-2s) following distance behavior.",
        ),
    ] = None
    following_distance_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="followingDistanceWeight"),
        pydantic.Field(alias="followingDistanceWeight", description="Score weight for following distance behavior."),
    ] = None
    forward_collision_warning_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="forwardCollisionWarningWeight"),
        pydantic.Field(
            alias="forwardCollisionWarningWeight", description="Score weight for forward collision warning behavior."
        ),
    ] = None
    harsh_accel_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="harshAccelWeight"),
        pydantic.Field(alias="harshAccelWeight", description="Score weight for harsh acceleration behavior."),
    ] = None
    harsh_brake_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="harshBrakeWeight"),
        pydantic.Field(alias="harshBrakeWeight", description="Score weight for harsh braking behavior."),
    ] = None
    harsh_turn_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="harshTurnWeight"),
        pydantic.Field(alias="harshTurnWeight", description="Score weight for harsh turn behavior."),
    ] = None
    heavy_speeding_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="heavySpeedingWeight"),
        pydantic.Field(alias="heavySpeedingWeight", description="Score weight for heavy speeding (20-30% over limit)."),
    ] = None
    inattentive_driving_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="inattentiveDrivingWeight"),
        pydantic.Field(alias="inattentiveDrivingWeight", description="Score weight for inattentive driving behavior."),
    ] = None
    lane_departure_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="laneDepartureWeight"),
        pydantic.Field(alias="laneDepartureWeight", description="Score weight for lane departure behavior."),
    ] = None
    late_response_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="lateResponseWeight"),
        pydantic.Field(alias="lateResponseWeight", description="Score weight for late response behavior."),
    ] = None
    light_speeding_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="lightSpeedingWeight"),
        pydantic.Field(alias="lightSpeedingWeight", description="Score weight for light speeding (0-10% over limit)."),
    ] = None
    max_speed_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="maxSpeedWeight"),
        pydantic.Field(alias="maxSpeedWeight", description="Score weight for max speed events."),
    ] = None
    mobile_usage_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="mobileUsageWeight"),
        pydantic.Field(alias="mobileUsageWeight", description="Score weight for mobile usage behavior."),
    ] = None
    moderate_speeding_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="moderateSpeedingWeight"),
        pydantic.Field(
            alias="moderateSpeedingWeight", description="Score weight for moderate speeding (10-20% over limit)."
        ),
    ] = None
    near_collision_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="nearCollisionWeight"),
        pydantic.Field(alias="nearCollisionWeight", description="Score weight for near collision behavior."),
    ] = None
    no_seatbelt_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="noSeatbeltWeight"),
        pydantic.Field(alias="noSeatbeltWeight", description="Score weight for no seatbelt behavior."),
    ] = None
    obstructed_camera_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="obstructedCameraWeight"),
        pydantic.Field(alias="obstructedCameraWeight", description="Score weight for obstructed camera behavior."),
    ] = None
    ran_red_light_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="ranRedLightWeight"),
        pydantic.Field(alias="ranRedLightWeight", description="Score weight for driver running red light."),
    ] = None
    rolling_stop_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="rollingStopWeight"),
        pydantic.Field(alias="rollingStopWeight", description="Score weight for rolling stop behavior."),
    ] = None
    severe_speeding_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="severeSpeedingWeight"),
        pydantic.Field(
            alias="severeSpeedingWeight", description="Score weight for severe speeding (over 30% over limit)."
        ),
    ] = None
    smoking_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="smokingWeight"),
        pydantic.Field(alias="smokingWeight", description="Score weight for smoking behavior."),
    ] = None
    speeding_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="speedingWeight"),
        pydantic.Field(alias="speedingWeight", description="Score weight for manual speeding event."),
    ] = None
    vulnerable_road_user_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="vulnerableRoadUserWeight"),
        pydantic.Field(alias="vulnerableRoadUserWeight", description="Score weight for vulnerable road user behavior."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .column_response_body import ColumnResponseBody
from .filters_response_body import FiltersResponseBody


class ReportConfigObjectResponseBody(UniversalBaseModel):
    columns: typing.List[ColumnResponseBody] = pydantic.Field()
    """
    Columns included in the report configuration.
    """

    dataset_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="datasetId"),
        pydantic.Field(alias="datasetId", description="ID of dataset for report configuration."),
    ]
    filters: typing.Optional[FiltersResponseBody] = None
    id: str = pydantic.Field()
    """
    ID of report configuration.
    """

    name: str = pydantic.Field()
    """
    Name of report configuration.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SafetyBehaviorTriggerDetailsObjectRequestBodyBehaviorsItem = typing.Union[
    typing.Literal[
        "Acceleration",
        "AggressiveDriving",
        "BluetoothHeadset",
        "Braking",
        "ContextConstructionOrWorkZone",
        "ContextSnowyOrIcy",
        "ContextVulnerableRoadUser",
        "ContextWet",
        "Crash",
        "CustomerCustomEvent1",
        "CustomerCustomEvent10",
        "CustomerCustomEvent2",
        "CustomerCustomEvent3",
        "CustomerCustomEvent4",
        "CustomerCustomEvent5",
        "CustomerCustomEvent6",
        "CustomerCustomEvent7",
        "CustomerCustomEvent8",
        "CustomerCustomEvent9",
        "DefensiveDriving",
        "DidNotYield",
        "Drinking",
        "Drowsy",
        "Eating",
        "EatingDrinking",
        "EdgeDistractedDriving",
        "EdgeRailroadCrossingViolation",
        "FollowingDistance",
        "FollowingDistanceModerate",
        "FollowingDistanceSevere",
        "ForwardCollisionWarning",
        "GenericDistraction",
        "GenericTailgating",
        "HarshTurn",
        "HeavySpeeding",
        "HighSpeedSuddenDisconnect",
        "HosViolation",
        "Idling",
        "Invalid",
        "LaneDeparture",
        "LateResponse",
        "LeftTurn",
        "LightSpeeding",
        "MaxSpeed",
        "MobileUsage",
        "ModerateSpeeding",
        "NearCollison",
        "NearPedestrianCollision",
        "NoSeatbelt",
        "ObstructedCamera",
        "OtherViolation",
        "Passenger",
        "PolicyViolationMask",
        "ProtectiveEquipment",
        "RanRedLight",
        "Reversing",
        "RollingStop",
        "RolloverProtection",
        "SamsaraCustomEvent1",
        "SamsaraCustomEvent10",
        "SamsaraCustomEvent2",
        "SamsaraCustomEvent3",
        "SamsaraCustomEvent4",
        "SamsaraCustomEvent5",
        "SamsaraCustomEvent6",
        "SamsaraCustomEvent7",
        "SamsaraCustomEvent8",
        "SamsaraCustomEvent9",
        "SevereSpeeding",
        "Smoking",
        "Speeding",
        "UTurn",
        "UnsafeManeuver",
        "UnsafeParking",
        "VehicleInBlindSpotWarning",
        "VulnerableRoadUserCollisionWarning",
        "YawControl",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.document_pdf_generation_response import DocumentPdfGenerationResponse
from ..types.document_pdf_query_response import DocumentPdfQueryResponse
from ..types.document_types_get_document_types_response_body import DocumentTypesGetDocumentTypesResponseBody
from ..types.documents_get_document_response_body import DocumentsGetDocumentResponseBody
from ..types.documents_get_documents_response_body import DocumentsGetDocumentsResponseBody
from ..types.documents_post_document_response_body import DocumentsPostDocumentResponseBody
from ..types.field_object_post_request_body import FieldObjectPostRequestBody
from .types.documents_post_document_request_body_state import DocumentsPostDocumentRequestBodyState

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDocumentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_document_types(
        self, *, after: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DocumentTypesGetDocumentTypesResponseBody]:
        """
        Returns a list of the organization document types. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getDriverDocumentTypesByOrgId).

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DocumentTypesGetDocumentTypesResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/document-types",
            method="GET",
            params={
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentTypesGetDocumentTypesResponseBody,
                    parse_obj_as(
                        type_=DocumentTypesGetDocumentTypesResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_documents(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        document_type_id: typing.Optional[str] = None,
        query_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DocumentsGetDocumentsResponseBody]:
        """
        Get all documents for the given time range. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getDriverDocumentsByOrgId).

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        document_type_id : typing.Optional[str]
            ID of the document template type.

        query_by : typing.Optional[str]
            Query by document creation time (`created`) or updated time (`updated`). Defaults to `created`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DocumentsGetDocumentsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/documents",
            method="GET",
            params={
                "startTime": start_time,
                "endTime": end_time,
                "after": after,
                "documentTypeId": document_type_id,
                "queryBy": query_by,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentsGetDocumentsResponseBody,
                    parse_obj_as(
                        type_=DocumentsGetDocumentsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def post_document(
        self,
        *,
        document_type_id: str,
        driver_id: str,
        fields: typing.Optional[typing.Sequence[FieldObjectPostRequestBody]] = OMIT,
        name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        route_stop_id: typing.Optional[str] = OMIT,
        state: typing.Optional[DocumentsPostDocumentRequestBodyState] = OMIT,
        vehicle_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DocumentsPostDocumentResponseBody]:
        """
        Creates a single document. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/createDriverDocument).

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        document_type_id : str
            ID for the document type.

        driver_id : str
            ID of the driver. Can be either unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        fields : typing.Optional[typing.Sequence[FieldObjectPostRequestBody]]
            The fields associated with this document.

        name : typing.Optional[str]
            Name of the document.

        notes : typing.Optional[str]
            Notes on the document.

        route_stop_id : typing.Optional[str]
            ID of the route stop. Can be either unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the route stop.

        state : typing.Optional[DocumentsPostDocumentRequestBodyState]
            The condition of the document created for the driver. Can be either `required` or `submitted`, if no value is specified, `state` defaults to `required`. `required` documents are pre-populated documents for the Driver to fill out in the Driver App.  Valid values: `submitted`, `required`

        vehicle_id : typing.Optional[str]
            ID of the vehicle. Can be either unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DocumentsPostDocumentResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/documents",
            method="POST",
            json={
                "documentTypeId": document_type_id,
                "driverId": driver_id,
                "fields": convert_and_respect_annotation_metadata(
                    object_=fields, annotation=typing.Sequence[FieldObjectPostRequestBody], direction="write"
                ),
                "name": name,
                "notes": notes,
                "routeStopId": route_stop_id,
                "state": state,
                "vehicleId": vehicle_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentsPostDocumentResponseBody,
                    parse_obj_as(
                        type_=DocumentsPostDocumentResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def generate_document_pdf(
        self, *, document_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DocumentPdfGenerationResponse]:
        """
        Request creation of a document PDF.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        document_id : str
            ID of the document.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DocumentPdfGenerationResponse]
            Newly created PDF generation job.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/documents/pdfs",
            method="POST",
            json={
                "documentId": document_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentPdfGenerationResponse,
                    parse_obj_as(
                        type_=DocumentPdfGenerationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_document_pdf(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DocumentPdfQueryResponse]:
        """
        Returns generation job status and download URL for a PDF.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the pdf.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DocumentPdfQueryResponse]
            Document PDF job status and download URL.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fleet/documents/pdfs/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentPdfQueryResponse,
                    parse_obj_as(
                        type_=DocumentPdfQueryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_document(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DocumentsGetDocumentResponseBody]:
        """
        Returns a single document. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getDriverDocumentByIdAndDriverId).

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the document

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DocumentsGetDocumentResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fleet/documents/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentsGetDocumentResponseBody,
                    parse_obj_as(
                        type_=DocumentsGetDocumentResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_document(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Deletes a single document. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/deleteDriverDocumentByIdAndDriverId).

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the document to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fleet/documents/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawDocumentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_document_types(
        self, *, after: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DocumentTypesGetDocumentTypesResponseBody]:
        """
        Returns a list of the organization document types. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getDriverDocumentTypesByOrgId).

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DocumentTypesGetDocumentTypesResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/document-types",
            method="GET",
            params={
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentTypesGetDocumentTypesResponseBody,
                    parse_obj_as(
                        type_=DocumentTypesGetDocumentTypesResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_documents(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        document_type_id: typing.Optional[str] = None,
        query_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DocumentsGetDocumentsResponseBody]:
        """
        Get all documents for the given time range. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getDriverDocumentsByOrgId).

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        document_type_id : typing.Optional[str]
            ID of the document template type.

        query_by : typing.Optional[str]
            Query by document creation time (`created`) or updated time (`updated`). Defaults to `created`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DocumentsGetDocumentsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/documents",
            method="GET",
            params={
                "startTime": start_time,
                "endTime": end_time,
                "after": after,
                "documentTypeId": document_type_id,
                "queryBy": query_by,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentsGetDocumentsResponseBody,
                    parse_obj_as(
                        type_=DocumentsGetDocumentsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def post_document(
        self,
        *,
        document_type_id: str,
        driver_id: str,
        fields: typing.Optional[typing.Sequence[FieldObjectPostRequestBody]] = OMIT,
        name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        route_stop_id: typing.Optional[str] = OMIT,
        state: typing.Optional[DocumentsPostDocumentRequestBodyState] = OMIT,
        vehicle_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DocumentsPostDocumentResponseBody]:
        """
        Creates a single document. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/createDriverDocument).

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        document_type_id : str
            ID for the document type.

        driver_id : str
            ID of the driver. Can be either unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        fields : typing.Optional[typing.Sequence[FieldObjectPostRequestBody]]
            The fields associated with this document.

        name : typing.Optional[str]
            Name of the document.

        notes : typing.Optional[str]
            Notes on the document.

        route_stop_id : typing.Optional[str]
            ID of the route stop. Can be either unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the route stop.

        state : typing.Optional[DocumentsPostDocumentRequestBodyState]
            The condition of the document created for the driver. Can be either `required` or `submitted`, if no value is specified, `state` defaults to `required`. `required` documents are pre-populated documents for the Driver to fill out in the Driver App.  Valid values: `submitted`, `required`

        vehicle_id : typing.Optional[str]
            ID of the vehicle. Can be either unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DocumentsPostDocumentResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/documents",
            method="POST",
            json={
                "documentTypeId": document_type_id,
                "driverId": driver_id,
                "fields": convert_and_respect_annotation_metadata(
                    object_=fields, annotation=typing.Sequence[FieldObjectPostRequestBody], direction="write"
                ),
                "name": name,
                "notes": notes,
                "routeStopId": route_stop_id,
                "state": state,
                "vehicleId": vehicle_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentsPostDocumentResponseBody,
                    parse_obj_as(
                        type_=DocumentsPostDocumentResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def generate_document_pdf(
        self, *, document_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DocumentPdfGenerationResponse]:
        """
        Request creation of a document PDF.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        document_id : str
            ID of the document.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DocumentPdfGenerationResponse]
            Newly created PDF generation job.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/documents/pdfs",
            method="POST",
            json={
                "documentId": document_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentPdfGenerationResponse,
                    parse_obj_as(
                        type_=DocumentPdfGenerationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_document_pdf(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DocumentPdfQueryResponse]:
        """
        Returns generation job status and download URL for a PDF.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the pdf.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DocumentPdfQueryResponse]
            Document PDF job status and download URL.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fleet/documents/pdfs/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentPdfQueryResponse,
                    parse_obj_as(
                        type_=DocumentPdfQueryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_document(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DocumentsGetDocumentResponseBody]:
        """
        Returns a single document. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getDriverDocumentByIdAndDriverId).

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the document

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DocumentsGetDocumentResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fleet/documents/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentsGetDocumentResponseBody,
                    parse_obj_as(
                        type_=DocumentsGetDocumentResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_document(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Deletes a single document. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/deleteDriverDocumentByIdAndDriverId).

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the document to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fleet/documents/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .goa_driver_tiny_response_response_body import GoaDriverTinyResponseResponseBody
from .goa_vehicle_tiny_response_response_body import GoaVehicleTinyResponseResponseBody
from .live_sharing_link_response_object_response_body import LiveSharingLinkResponseObjectResponseBody
from .route_settings_response_body import RouteSettingsResponseBody
from .routes_stop_response_object_response_body import RoutesStopResponseObjectResponseBody


class BaseRouteResponseObjectResponseBody(UniversalBaseModel):
    actual_route_end_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="actualRouteEndTime"),
        pydantic.Field(
            alias="actualRouteEndTime", description="Actual end time, if it exists, for the route in RFC 3339 format."
        ),
    ] = None
    actual_route_start_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="actualRouteStartTime"),
        pydantic.Field(
            alias="actualRouteStartTime",
            description="Actual start time, if it exists, for the route in RFC 3339 format.",
        ),
    ] = None
    driver: typing.Optional[GoaDriverTinyResponseResponseBody] = None
    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    id: str = pydantic.Field()
    """
    ID of the route
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Route name
    """

    notes: typing.Optional[str] = pydantic.Field(default=None)
    """
    Notes for the route
    """

    org_local_timezone: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="orgLocalTimezone"),
        pydantic.Field(
            alias="orgLocalTimezone",
            description="The local timezone, as defined in Settings for your organization. Timezones use [IANA timezone database](https://www.iana.org/time-zones) keys (e.g. `America/Los_Angeles`, `America/New_York`, `Europe/London`, etc.). You can find a mapping of common timezone formats to IANA timezone keys [here](https://unicode.org/cldr/charts/latest/supplemental/zone_tzid.html).",
        ),
    ] = None
    recurring_route_live_sharing_links: typing_extensions.Annotated[
        typing.Optional[typing.List[LiveSharingLinkResponseObjectResponseBody]],
        FieldMetadata(alias="recurringRouteLiveSharingLinks"),
        pydantic.Field(
            alias="recurringRouteLiveSharingLinks",
            description="List of shareable, non-expired 'By recurring Route' Live Sharing Links.",
        ),
    ] = None
    scheduled_route_end_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="scheduledRouteEndTime"),
        pydantic.Field(
            alias="scheduledRouteEndTime",
            description="Scheduled end time, if it exists, for the route in RFC 3339 format.",
        ),
    ] = None
    scheduled_route_start_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="scheduledRouteStartTime"),
        pydantic.Field(
            alias="scheduledRouteStartTime",
            description="Scheduled start time, if it exists, for the route in RFC 3339 format.",
        ),
    ] = None
    settings: typing.Optional[RouteSettingsResponseBody] = None
    stops: typing.Optional[typing.List[RoutesStopResponseObjectResponseBody]] = pydantic.Field(default=None)
    """
    List of stops along the route
    """

    tag_ids: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="tagIds"),
        pydantic.Field(alias="tagIds", description="An array of IDs of tags to associate with this route."),
    ] = None
    vehicle: typing.Optional[GoaVehicleTinyResponseResponseBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

HosViolationObjectResponseBodyType = typing.Union[
    typing.Literal[
        "NONE",
        "californiaMealbreakMissed",
        "cycleHoursOn",
        "cycleOffHoursAfterOnDutyHours",
        "dailyDrivingHours",
        "dailyOffDutyDeferralAddToDay2Consecutive",
        "dailyOffDutyDeferralNotPartMandatory",
        "dailyOffDutyDeferralTwoDayDrivingLimit",
        "dailyOffDutyDeferralTwoDayOffDuty",
        "dailyOffDutyNonResetHours",
        "dailyOffDutyTotalHours",
        "dailyOnDutyHours",
        "mandatory24HoursOffDuty",
        "restbreakMissed",
        "shiftDrivingHours",
        "shiftHours",
        "shiftOnDutyHours",
        "unsubmittedLogs",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

FormSubmissionRequestFieldInputObjectRequestBodyType = typing.Union[
    typing.Literal[
        "number", "text", "multiple_choice", "check_boxes", "datetime", "asset", "person", "table", "geofence"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .uploaded_media_object_response_body_camera_role import UploadedMediaObjectResponseBodyCameraRole
from .uploaded_media_object_response_body_input import UploadedMediaObjectResponseBodyInput
from .uploaded_media_object_response_body_media_type import UploadedMediaObjectResponseBodyMediaType
from .uploaded_media_object_response_body_trigger_reason import UploadedMediaObjectResponseBodyTriggerReason
from .url_info_object_response_body import UrlInfoObjectResponseBody


class UploadedMediaObjectResponseBody(UniversalBaseModel):
    available_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="availableAtTime"),
        pydantic.Field(
            alias="availableAtTime",
            description="Timestamp, in RFC 3339 format, at which the media item was made available. Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00",
        ),
    ]
    camera_role: typing_extensions.Annotated[
        typing.Optional[UploadedMediaObjectResponseBodyCameraRole],
        FieldMetadata(alias="cameraRole"),
        pydantic.Field(
            alias="cameraRole",
            description="Camera role for this media.  Valid values: `leftMirrorMount`, `leftSide`, `rightMirrorMount`, `rightSide`, `rearHigh`, `rearBumper`, `inCab`, `front`, `hopper`, `other1`, `other2`, `other3`, `other4`, `leftBev`, `rightBev`, `rearBev`, `frontBev`, `otherBev`, `bevNotUsed`",
        ),
    ] = None
    end_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="endTime"),
        pydantic.Field(
            alias="endTime",
            description=" An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).",
        ),
    ]
    input: UploadedMediaObjectResponseBodyInput = pydantic.Field()
    """
    Input type for this media.  Valid values: `dashcamForwardFacing`, `dashcamInwardFacing`, `analog1`, `analog2`, `analog3`, `analog4`
    """

    media_type: typing_extensions.Annotated[
        UploadedMediaObjectResponseBodyMediaType,
        FieldMetadata(alias="mediaType"),
        pydantic.Field(
            alias="mediaType",
            description="Type of media.  Valid values: `image`, `videoHighRes`, `videoLowRes`, `hyperlapse`",
        ),
    ]
    start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="startTime"),
        pydantic.Field(
            alias="startTime",
            description=" A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).",
        ),
    ]
    trigger_reason: typing_extensions.Annotated[
        UploadedMediaObjectResponseBodyTriggerReason,
        FieldMetadata(alias="triggerReason"),
        pydantic.Field(
            alias="triggerReason",
            description="Trigger reason for this media capture.  Valid values: `api`, `panicButton`, `periodicStill`, `rfidEvent`, `safetyEvent`, `tripEndStill`, `tripStartStill`, `videoRetrieval`",
        ),
    ]
    url_info: typing_extensions.Annotated[
        typing.Optional[UrlInfoObjectResponseBody], FieldMetadata(alias="urlInfo"), pydantic.Field(alias="urlInfo")
    ] = None
    vehicle_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="vehicleId"),
        pydantic.Field(alias="vehicleId", description="Vehicle ID for which this media was captured. Examples: 1234"),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class RidershipRouteSetupPassengerObjectResponseBody(UniversalBaseModel):
    """
    A passenger assignment within a route setup.
    """

    drop_off_stop_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="dropOffStopId"),
        pydantic.Field(alias="dropOffStopId", description="The stop ID for the passenger's drop-off."),
    ] = None
    passenger_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="passengerId"),
        pydantic.Field(alias="passengerId", description="The Samsara UUID of the passenger."),
    ]
    pick_up_stop_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="pickUpStopId"),
        pydantic.Field(alias="pickUpStopId", description="The stop ID for the passenger's pick-up."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .action_object_response_body import ActionObjectResponseBody
from .operational_settings_object_response_body import OperationalSettingsObjectResponseBody
from .scope_object_response_body import ScopeObjectResponseBody
from .workflow_trigger_object_response_body import WorkflowTriggerObjectResponseBody


class PatchResponseWorkflowConfigurationObjectResponseBody(UniversalBaseModel):
    """
    The configuration of a alert.
    """

    actions: typing.List[ActionObjectResponseBody] = pydantic.Field()
    """
    An array of actions.
    """

    created_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime", description="The time the configuration was created in RFC 3339 format."),
    ]
    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    id: str = pydantic.Field()
    """
    The unqiue Samsara id of the alert configuration.
    """

    is_enabled: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isEnabled"),
        pydantic.Field(alias="isEnabled", description="Whether the alert is enabled or not."),
    ]
    last_modified_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="lastModifiedAtTime"),
        pydantic.Field(
            alias="lastModifiedAtTime", description="The time the configuration was last modified in RFC 3339 format."
        ),
    ]
    name: str = pydantic.Field()
    """
    The custom name of the configuration.
    """

    operational_settings: typing_extensions.Annotated[
        typing.Optional[OperationalSettingsObjectResponseBody],
        FieldMetadata(alias="operationalSettings"),
        pydantic.Field(alias="operationalSettings"),
    ] = None
    scope: ScopeObjectResponseBody
    triggers: typing.List[WorkflowTriggerObjectResponseBody] = pydantic.Field()
    """
    An array of triggers.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_vision_step_results_item_fixture_coordinates import V1VisionStepResultsItemFixtureCoordinates


class V1VisionStepResultsItemFixture(UniversalBaseModel):
    coordinates: typing.Optional[V1VisionStepResultsItemFixtureCoordinates] = None
    found: typing.Optional[bool] = None
    rotation_degrees: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="rotationDegrees"), pydantic.Field(alias="rotationDegrees")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SpeedingSettingsObjectResponseBodyUnit = typing.Union[
    typing.Literal["milesPerHour", "kilometersPerHour", "percentage"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.auth_token_auth_token_response_body import AuthTokenAuthTokenResponseBody
from .raw_client import AsyncRawAuthTokenForDriverClient, RawAuthTokenForDriverClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AuthTokenForDriverClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAuthTokenForDriverClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAuthTokenForDriverClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAuthTokenForDriverClient
        """
        return self._raw_client

    def auth_token(
        self,
        *,
        code: str,
        driver_id: typing.Optional[int] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        username: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AuthTokenAuthTokenResponseBody:
        """
        Creates a short-lived auth token a driver can use to login.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Driver Auth Token** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        code : str
            Required. Random 12+ character string, used with the auth token to help secure the client from intercepted tokens.

        driver_id : typing.Optional[int]
            Optional. Samsara ID of the driver. One of `id`, `externalId`, or `username` is required.

        external_id : typing.Optional[str]
            Optional. External ID of the driver, in the format `key:value` (e.g., `payrollId:ABFS18600`). One of `id`, `externalId`, or `username` is required.

        username : typing.Optional[str]
            Optional. Username of the driver. This is the login identifier configured when the driver is created. One of `id`, `externalId`, or `username` is required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthTokenAuthTokenResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.auth_token_for_driver.auth_token(
            code="dp[gZc1wAigz4uGa0Hh",
        )
        """
        _response = self._raw_client.auth_token(
            code=code, driver_id=driver_id, external_id=external_id, username=username, request_options=request_options
        )
        return _response.data


class AsyncAuthTokenForDriverClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAuthTokenForDriverClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAuthTokenForDriverClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAuthTokenForDriverClient
        """
        return self._raw_client

    async def auth_token(
        self,
        *,
        code: str,
        driver_id: typing.Optional[int] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        username: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AuthTokenAuthTokenResponseBody:
        """
        Creates a short-lived auth token a driver can use to login.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Driver Auth Token** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        code : str
            Required. Random 12+ character string, used with the auth token to help secure the client from intercepted tokens.

        driver_id : typing.Optional[int]
            Optional. Samsara ID of the driver. One of `id`, `externalId`, or `username` is required.

        external_id : typing.Optional[str]
            Optional. External ID of the driver, in the format `key:value` (e.g., `payrollId:ABFS18600`). One of `id`, `externalId`, or `username` is required.

        username : typing.Optional[str]
            Optional. Username of the driver. This is the login identifier configured when the driver is created. One of `id`, `externalId`, or `username` is required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthTokenAuthTokenResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.auth_token_for_driver.auth_token(
                code="dp[gZc1wAigz4uGa0Hh",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.auth_token(
            code=code, driver_id=driver_id, external_id=external_id, username=username, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class TachographVehicleFile(UniversalBaseModel):
    """
    Tachograph vehicle file
    """

    created_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(
            alias="createdAtTime",
            description="Creation time of files in RFC 3339 format. This is either the download time from the tachograph itself (for files downloaded via Samsara VG) or upload time (for files manually uploaded via Samsara UI).",
        ),
    ] = None
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the file.
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    A temporary URL which can be used to download the file. The link can be used multiple times and expires after an hour.
    """

    vehicle_identification_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="vehicleIdentificationNumber"),
        pydantic.Field(alias="vehicleIdentificationNumber", description="VIN associated with the vehicle file."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class AuthTokenForDriverResponseResponseBody(UniversalBaseModel):
    """
    Response object for create driver auth token
    """

    expiration_time: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="expirationTime"),
        pydantic.Field(
            alias="expirationTime",
            description="Expiration time of the token in Unix milliseconds since epoch. Clients must redeem the token before this timestamp.",
        ),
    ]
    token: str = pydantic.Field()
    """
    A one-time-use authentication token. Must be paired with the original code and driver identity in a separate request to exchange for a session.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .driver_home_terminal_address import DriverHomeTerminalAddress
from .driver_home_terminal_name import DriverHomeTerminalName


class DriverCarrierSettings(UniversalBaseModel):
    """
    Carrier for a given driver. If the driver's carrier differs from the general organization's carrier settings, the override value is used. Updating this value only updates the override setting for this driver.
    """

    carrier_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="carrierName"),
        pydantic.Field(alias="carrierName", description="Carrier for a given driver."),
    ] = None
    dot_number: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="dotNumber"),
        pydantic.Field(
            alias="dotNumber",
            description="Carrier US DOT Number. If this differs from the general organization's settings, the override value is used. Updating this value only updates the override setting for this driver.",
        ),
    ] = None
    home_terminal_address: typing_extensions.Annotated[
        typing.Optional[DriverHomeTerminalAddress],
        FieldMetadata(alias="homeTerminalAddress"),
        pydantic.Field(alias="homeTerminalAddress"),
    ] = None
    home_terminal_name: typing_extensions.Annotated[
        typing.Optional[DriverHomeTerminalName],
        FieldMetadata(alias="homeTerminalName"),
        pydantic.Field(alias="homeTerminalName"),
    ] = None
    main_office_address: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="mainOfficeAddress"),
        pydantic.Field(
            alias="mainOfficeAddress",
            description="Main office address for a given driver. If this differs from the general organization's settings, the override value is used. ",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_asset_reefer_response_reefer_stats import V1AssetReeferResponseReeferStats


class V1AssetReeferResponse(UniversalBaseModel):
    """
    Reefer-specific asset details
    """

    asset_type: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="assetType"),
        pydantic.Field(alias="assetType", description="Asset type"),
    ] = None
    id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Asset ID
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Asset name
    """

    reefer_stats: typing_extensions.Annotated[
        typing.Optional[V1AssetReeferResponseReeferStats],
        FieldMetadata(alias="reeferStats"),
        pydantic.Field(alias="reeferStats"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_trip_response_end_address import V1TripResponseEndAddress
from .v_1_trip_response_end_coordinates import V1TripResponseEndCoordinates
from .v_1_trip_response_start_address import V1TripResponseStartAddress
from .v_1_trip_response_start_coordinates import V1TripResponseStartCoordinates


class V1TripResponseTrips(UniversalBaseModel):
    asset_ids: typing_extensions.Annotated[
        typing.Optional[typing.List[int]],
        FieldMetadata(alias="assetIds"),
        pydantic.Field(alias="assetIds", description="List of associated asset IDs"),
    ] = None
    codriver_ids: typing_extensions.Annotated[
        typing.Optional[typing.List[int]],
        FieldMetadata(alias="codriverIds"),
        pydantic.Field(alias="codriverIds", description="List of codriver IDs"),
    ] = None
    distance_meters: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="distanceMeters"),
        pydantic.Field(
            alias="distanceMeters",
            description="Length of the trip in meters. This value is calculated from the GPS data collected by the Samsara Vehicle Gateway.",
        ),
    ] = None
    driver_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="driverId"),
        pydantic.Field(alias="driverId", description="ID of the driver."),
    ] = None
    end_address: typing_extensions.Annotated[
        typing.Optional[V1TripResponseEndAddress], FieldMetadata(alias="endAddress"), pydantic.Field(alias="endAddress")
    ] = None
    end_coordinates: typing_extensions.Annotated[
        typing.Optional[V1TripResponseEndCoordinates],
        FieldMetadata(alias="endCoordinates"),
        pydantic.Field(alias="endCoordinates"),
    ] = None
    end_location: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="endLocation"),
        pydantic.Field(
            alias="endLocation", description="Geocoded street address of start (latitude, longitude) coordinates."
        ),
    ] = None
    end_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="endMs"),
        pydantic.Field(
            alias="endMs",
            description="End of the trip in UNIX milliseconds. Ongoing trips are indicated by an endMs value of 9223372036854775807.",
        ),
    ] = None
    end_odometer: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="endOdometer"),
        pydantic.Field(
            alias="endOdometer",
            description="Odometer reading (in meters) at the end of the trip. This is read from the vehicle's on-board diagnostics. If Samsara cannot read the vehicle's odometer values from on-board diagnostics, this value will be 0.",
        ),
    ] = None
    fuel_consumed_ml: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="fuelConsumedMl"),
        pydantic.Field(alias="fuelConsumedMl", description="Amount in milliliters of fuel consumed on this trip."),
    ] = None
    start_address: typing_extensions.Annotated[
        typing.Optional[V1TripResponseStartAddress],
        FieldMetadata(alias="startAddress"),
        pydantic.Field(alias="startAddress"),
    ] = None
    start_coordinates: typing_extensions.Annotated[
        typing.Optional[V1TripResponseStartCoordinates],
        FieldMetadata(alias="startCoordinates"),
        pydantic.Field(alias="startCoordinates"),
    ] = None
    start_location: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="startLocation"),
        pydantic.Field(
            alias="startLocation", description="Geocoded street address of start (latitude, longitude) coordinates."
        ),
    ] = None
    start_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="startMs"),
        pydantic.Field(alias="startMs", description="Beginning of the trip in UNIX milliseconds."),
    ] = None
    start_odometer: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="startOdometer"),
        pydantic.Field(
            alias="startOdometer",
            description="Odometer reading (in meters) at the beginning of the trip. This is read from the vehicle's on-board diagnostics. If Samsara cannot read the vehicle's odometer values from on-board diagnostics, this value will be 0.",
        ),
    ] = None
    toll_meters: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="tollMeters"),
        pydantic.Field(alias="tollMeters", description="Length in meters trip spent on toll roads."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .fft_spectra_data_point import FftSpectraDataPoint
from .j_1939_d_1_status_data_point import J1939D1StatusDataPoint
from .location_data_point import LocationDataPoint
from .number_data_point import NumberDataPoint
from .string_data_point import StringDataPoint


class DataInputResponseAllOf(UniversalBaseModel):
    fft_spectra_points: typing_extensions.Annotated[
        typing.Optional[typing.List[FftSpectraDataPoint]],
        FieldMetadata(alias="fftSpectraPoints"),
        pydantic.Field(alias="fftSpectraPoints", description="List of FFT spectra data points from the data input."),
    ] = None
    j_1939_d_1_status_points: typing_extensions.Annotated[
        typing.Optional[typing.List[J1939D1StatusDataPoint]],
        FieldMetadata(alias="j1939D1StatusPoints"),
        pydantic.Field(alias="j1939D1StatusPoints", description="List of active J1939D1 statuses."),
    ] = None
    location_points: typing_extensions.Annotated[
        typing.Optional[typing.List[LocationDataPoint]],
        FieldMetadata(alias="locationPoints"),
        pydantic.Field(alias="locationPoints", description="List of location data points from the data input."),
    ] = None
    number_points: typing_extensions.Annotated[
        typing.Optional[typing.List[NumberDataPoint]],
        FieldMetadata(alias="numberPoints"),
        pydantic.Field(alias="numberPoints", description="List of numeric data points from the data input."),
    ] = None
    string_points: typing_extensions.Annotated[
        typing.Optional[typing.List[StringDataPoint]],
        FieldMetadata(alias="stringPoints"),
        pydantic.Field(alias="stringPoints", description="List of string data points from the data input."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.routes_get_route_events_stream_response_body import RoutesGetRouteEventsStreamResponseBody
from .raw_client import AsyncRawRouteEventsClient, RawRouteEventsClient


class RouteEventsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawRouteEventsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawRouteEventsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawRouteEventsClient
        """
        return self._raw_client

    def get_route_events_stream(
        self,
        *,
        after: typing.Optional[str] = None,
        start_time: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        include_external_ids: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RoutesGetRouteEventsStreamResponseBody:
        """
        Subscribes to a feed of immutable, append-only updates for routes. The initial request to this feed endpoint returns a cursor, which can be used in subsequent requests to fetch events on all routes that occurred after the last retrieved event. An initial request (with empty after and startTime parameters) will return events from the past 24 hours.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        start_time : typing.Optional[str]
            A start time in RFC 3339 format. Defaults to 24 hours ago if not provided. Millisecond precision and timezones are supported. (Examples: 2025-07-01T19:08:25Z, 2025-07-01T19:08:25.455Z, OR 2025-07-01T14:00:12-04:00). Cannot be more than 7 days in the past.

        limit : typing.Optional[int]
            The limit for how many events will be in the response. Default and max for this value is 200 events.

        include_external_ids : typing.Optional[bool]
            Optional boolean indicating whether to return external IDs on supported entities

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RoutesGetRouteEventsStreamResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.route_events.get_route_events_stream()
        """
        _response = self._raw_client.get_route_events_stream(
            after=after,
            start_time=start_time,
            limit=limit,
            include_external_ids=include_external_ids,
            request_options=request_options,
        )
        return _response.data


class AsyncRouteEventsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawRouteEventsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawRouteEventsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawRouteEventsClient
        """
        return self._raw_client

    async def get_route_events_stream(
        self,
        *,
        after: typing.Optional[str] = None,
        start_time: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        include_external_ids: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RoutesGetRouteEventsStreamResponseBody:
        """
        Subscribes to a feed of immutable, append-only updates for routes. The initial request to this feed endpoint returns a cursor, which can be used in subsequent requests to fetch events on all routes that occurred after the last retrieved event. An initial request (with empty after and startTime parameters) will return events from the past 24 hours.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Routes** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        start_time : typing.Optional[str]
            A start time in RFC 3339 format. Defaults to 24 hours ago if not provided. Millisecond precision and timezones are supported. (Examples: 2025-07-01T19:08:25Z, 2025-07-01T19:08:25.455Z, OR 2025-07-01T14:00:12-04:00). Cannot be more than 7 days in the past.

        limit : typing.Optional[int]
            The limit for how many events will be in the response. Default and max for this value is 200 events.

        include_external_ids : typing.Optional[bool]
            Optional boolean indicating whether to return external IDs on supported entities

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RoutesGetRouteEventsStreamResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.route_events.get_route_events_stream()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_route_events_stream(
            after=after,
            start_time=start_time,
            limit=limit,
            include_external_ids=include_external_ids,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_asset_reefer_response_reefer_stats_alarms import V1AssetReeferResponseReeferStatsAlarms


class V1AssetReeferResponseReeferStatsAlarms1(UniversalBaseModel):
    alarms: typing.Optional[typing.List[V1AssetReeferResponseReeferStatsAlarms]] = None
    changed_at_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="changedAtMs"),
        pydantic.Field(
            alias="changedAtMs", description="Timestamp when the alarms were reported, in Unix milliseconds since epoch"
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
